
#undef TRACE_SYSTEM
#define TRACE_SYSTEM kotrace

#if !defined (_KOTRACE_H) || defined(TRACE_HEADER_MULTI_READ)
#define _KOTRACE_H

#include <linux/tracepoint.h>

// Hack to allow a trace event without parameters which is useful since
// the trace output will include times
TRACE_EVENT(kotrace_print,                     \
    TP_PROTO(int dummy),                       \
    TP_ARGS(dummy),                            \
    TP_STRUCT__entry(                          \
      __field(int, dummy);                     \
    ),                                         \
    TP_fast_assign(                            \
      __entry->dummy = dummy;                  \
    ),                                         \
    TP_printk("kotrace: %d",  __entry->dummy));

#endif

#undef TRACE_INCLUDE_PATH
#undef TRACE_INCLUDE_FILE
#define TRACE_INCLUDE_PATH .
#define TRACE_INCLUDE_FILE kotrace
#include <trace/define_trace.h>
